
{% if structured_output %}
Give your answer in a pure JSON format with the following sections.
The content of each section should be formatted with markdown:

{
{% for title, description in sections.items() %}
  "{{ title }}": "{{ description | replace("\n", "\\n") }}",{% endfor %}
}

You MUST set a section as `null` if:
- It's not relevant to the investigation
- It does not contain relevant information
- Its content is not related to a possible root cause

<DO NOT list tools used and DO NOT add a `Tools` section>
{% else %}
Give your answer returning a markdown document structured with the following sections. Use top level markdown titles format `#`.
Ignore any section that is not relevant to the investigation.

{% for title, description in sections.items() %}
# {{ title }}
{{ description }}
{% endfor %}

# <DO NOT list tools used and DO NOT add a `# Tools` section>

You MUST ignore a section and skip it:
- It's not relevant to the investigation
- It does not contain relevant information
- Its content is not related to a possible root cause
{% endif %}
